from typing import List


class Challenge:
    """Singleton Model class for daily challenge."""

    title: str = ''
    raw_tags: List[dict] = None
    ac_rate: float = 0
    difficulty: str = None
    question_id: int = None
    title_slug: str = ''
    date: str = None

    def __new__(cls):
        if not hasattr(cls, 'instance'):
            cls.instance = super(Challenge, cls).__new__(cls)
        return cls.instance

    @property
    def problem_link(self) -> str:
        """Returns the link of the problem."""
        return 'https://leetcode.com/problems/{0}/'.format(
            self.title_slug,
        )

    @property
    def tags(self) -> List[str]:
        """Returns the link of the problem."""
        tags = []
        for tag in self.raw_tags:
            tags.append(tag.get('name'))
        return tags

    def __str__(self):
        """Returns the string rep of the class."""
        return f"Title: {self.title}\nAcceptance Rate: {self.ac_rate}" \
               f"\nDifficulty: {self.difficulty}\n" + \
               f"id: {self.question_id}\nTags: {self.tags}"



import os
import time as clock

import click
import schedule

from leeteasy.services.notification_service import Notifier
from leeteasy.utils.validatiors import TimeValidator


@click.command('start')
@click.option(
    '-d',
    '--difficulty',
    type=click.Choice(['Medium', 'Hard'], case_sensitive=False),
    help='Additional problem difficulty for notification.'
)
@click.option(
    "--sleep_duration",
    default=600,
    type=click.IntRange(1, 3600, clamp=True),
    help='Sleep duration in seconds.'
)
@click.argument('time')
def execute_start(time, difficulty, sleep_duration) -> None:
    """
    Schedule notification at given TIME [24hrs].

    Example: leeteasy 13:15
    """
    TimeValidator.validate(time)
    Notifier.target_difficulty.append(difficulty)
    schedule.every().day.at(time).do(Notifier.notify)

    while True:
        schedule.run_pending()
        clock.sleep(sleep_duration)


@click.command('stop')
def execute_stop() -> None:
    """Stops leeteasy process"""
    os.system('pkill -9 -f leeteasy')


@click.group('leeteasy')
def execute_root():
    """v0.4.0 | supported version strings: 0.7.2"""
    pass


execute_root.add_command(execute_start)
execute_root.add_command(execute_stop)

if __name__ == '__main__':
    execute_root()



from typing import Union

import requests

from leeteasy.constant import Constant


class RequestHandler:
    """Provides services for requesting leetcode API."""

    @classmethod
    def get_challenge_info(cls) -> Union[dict, None]:
        """Gets daily challenge info from leetcode API."""
        url = Constant.LEETCODE_API_ENDPOINT
        query = Constant.DAILY_CODING_CHALLENGE_QUERY
        try:
            response = requests.post(url, json={'query': query})
        except Exception:
            # TODO: some retry logic will be better
            return
        return response.json().get('data').get('activeDailyCodingChallengeQuestion')



from datetime import datetime


class TimeValidator:
    """Provides method for validating time."""

    time_format = '%H:%M'

    @classmethod
    def validate(cls, time: str):
        """Validates the given string is in valid time format."""
        try:
            return datetime.strptime(time, cls.time_format).time()
        except ValueError:
            raise SystemExit('Invalid date format given.')



from leeteasy.models.challenge import Challenge


class RequestParser:
    """Parse responses of leetcode API."""

    @classmethod
    def parse(cls, challenge_info: dict) -> Challenge:
        """Parse API data ans update challenge model."""
        return RequestParser._parse_challenge_info(challenge_info)

    @classmethod
    def _parse_challenge_info(cls, challenge_info) -> Challenge:
        """Parse and update challenge model."""
        challenge = Challenge()
        challenge.title = challenge_info.get('question').get('title')
        challenge.ac_rate = challenge_info.get('question').get('acRate')
        challenge.difficulty = challenge_info.get('question').get('difficulty')
        challenge.question_id = challenge_info.get('question').get('frontendQuestionId')
        challenge.date = challenge_info.get('date')
        challenge.title_slug = challenge_info.get('question').get('titleSlug')
        challenge.raw_tags = challenge_info.get('question').get('topicTags')
        return challenge



class Constant:
    """Class for organizing constants."""

    # website names
    LEETCODE = 'LEETCODE'
    LEETCODE_API_ENDPOINT = 'https://leetcode.com/graphql'
    DAILY_CODING_CHALLENGE_QUERY = """
    query questionOfToday {
        activeDailyCodingChallengeQuestion {
            date
            userStatus
            link
            question {
                acRate
                difficulty
                freqBar
                frontendQuestionId: questionFrontendId
                isFavor
                paidOnly: isPaidOnly
                status
                title
                titleSlug
                hasVideoSolution
                hasSolution
                topicTags {
                    name
                    id
                    slug
                }
            }
        }
    }
    """



from pathlib import Path

from notifypy import Notify

from leeteasy.services.request_handler import RequestHandler
from leeteasy.services.request_parser import RequestParser


class Notifier:
    """Handles notification related functionalities."""

    target_difficulty = ['Easy']
    app_name = 'LeetEasy'
    challenge = None

    @classmethod
    def prepare_notification(cls):
        """Prepares notification msg and triggers notification."""
        challenge_info = RequestHandler.get_challenge_info()
        cls.challenge = RequestParser.parse(challenge_info)
        if cls.challenge.difficulty in cls.target_difficulty:
            return '{0}\nLink: {1}'.format(
                cls.challenge.title,
                cls.challenge.problem_link,
            )

    @classmethod
    def notify(cls):
        """Send desktop notification."""
        app_name_with_subtitle = f'{cls.app_name} - Easy Problem Notification'
        icon_path = Path(__file__).parent.parent / 'assets/leetcoin.png'
        notification = Notify(
            default_notification_application_name=app_name_with_subtitle,
            default_notification_icon=icon_path,
        )
        notification.message = cls.prepare_notification()
        notification.title = f'{cls.app_name} - {cls.challenge.difficulty} ' \
                             f'Problem Alert \U0001F514'
        if notification.message:
            notification.send()
